#!/usr/bin/env python3

"""
NAME:
	dnsdeps.py - generate .depend for DNS

SYNOPSIS:
	dnsdeps.py [-N] "named_conf"

DESCRIPTION:
	Starting with the files on the command line
	recursively find dependencies and output to '.depend'.

	Also any files that end in '.soa' are added to the
	dependencies of '.zones' which is how 'dnsmagic.mk'
	drives 'updsoa.py'.

	More importantly, all files included however indirectly by a
	'.soa' file are added to the dependencies of that file.
	This ensures that make(1) will update the '.soa' when any of
	them are newer.

	Options:

	-N	Do not append dependencies to makefile.
		This option is for backwards compatability only,
		we do not support using any make(1) so old that
		it requires appending to makefile.

AUTHOR:
	Simon J. Gerraty <sjg@crufty.net>

"""

"""
RCSid:
	$Id: dnsdeps.py,v 1.11 2023/06/10 17:03:46 sjg Exp $

	@(#) Copyright (c) 1997-2023 Simon J. Gerraty.

	This file is provided in the hope that it will
	be of use.  There is absolutely NO WARRANTY.
	Permission to copy, redistribute or otherwise
	use this file is hereby granted provided that 
	the above copyright notice and this notice are
	left intact. 
      
	Please send copies of changes and bug-fixes to:
	sjg@crufty.net
"""


import sys
import os
import dnsutils
import conf as cf

Depends = {}

def get_depends(f, conf, soa=None):
    global Depends

    if f.endswith('.soa'):
        soa=f
        Depends['.zones'].append(f)
    Depends[f] = []

    try:
        for line in open(f, 'r').readlines():
            w = line.strip().split()
            if len(w) < 2:
                continue
            if w[0] in ['file', '$INCLUDE', 'include']:
                nf = w[1].strip(';').strip('"')
                if not nf in Depends[f]:
                    Depends[f].append(nf)
                    if soa and not nf in Depends[soa]:
                        Depends[soa].append(nf)
                    get_depends(nf, conf, soa)
    except PermissionError as e:
        if 'key' in f:
            pass
    except FileNotFoundError as e:
        if f.endswith('.soa'):
            # we know it will depend on this
            Depends[f].append(f.strip('.soa')+'.db')
        pass
        
def myopts(o,a,conf):

    if o == '-C':
        os.chdir(a)
    elif o == '-N':
        conf['append_makefile'] = False

if __name__ == '__main__':

    conf = {
        'ns_list': [],
        'depend_file': '.depend',
        }

    conf,args = dnsutils.app_init('C:N', myopts, conf)
    Depends['.zones'] = []
    for a in args:
        get_depends(a, conf)

    depend = open(conf['depend_file'], 'w')
    for k,v in Depends.items():
        print('{}: \\'.format(k), file=depend)
        for d in v:
            print('\t{} \\'.format(d), file=depend)
        print('', file=depend)
    depend.close()
    sys.exit(0)
