#!/usr/bin/env python3

"""
NAME:
	getdata.py - extract SOA and A records from DNS dump

SYNOPSIS:
	getdata.py "named_dump"

DESCRIPTION:
	This script reads "named_dump" and outputs SOA records for
	each domain followed by PTR,NS,MX,A and AAAA records.
	The output looks like::

		"type key data"

	for example::

		SOA crufty.net ns.crufty.net hostmaster@crufty.net
		SOA 250.12.203.in-addr.arpa ns.crufty.net
		PTR 1.250.12.203.in-addr.arpa gate.crufty.net
		PTR 130.250.12.203.in-addr.arpa gate.crufty.net
		NS crufty.net ns.crufty.net
		MX crufty.net gate.crufty.net 100
		A ns.crufty.net 203.12.250.1
		A gate.crufty.net 203.12.250.1
		A gate.crufty.net 203.12.250.130

	Which makes further automated processing such as generating
	reverse maps, quite simple.

	Options:
        
	-c "config"
		Read settings from "config", see conf.py for syntax.
		We will automatically read 'dns.cf' and '${progname}.cf'
		from the directory this script is in ('${progdir}'),
		as well as '.dnc.cf' and '.${progname}.cf' if they
		exist in the current directory or a parent.

		In fact unlike earlier iterations of this tool, we
		expect most settings to be provided via a "config" file.

	-d "dbg"
		Enable debug output for the specified operations.

	-o "var=value"
		This allows overriding random settings from "config".

	-X "type"
		For backwards compatability.  Adds a "type" to
		'want_records'.  This can also/better be done
		via the "config" files eg.::

			want_records [+=] cname,txt

		See conf.py for syntax.

NOTES:
	To simplify down stream processing "type" is always in UPPER
	case and all other output is in lower case.  We attempt to
	ignore bogus data especially in PTR records.

AUTHOR:
	Simon J. Gerraty <sjg@crufty.net>

"""

"""
RCSid:
	$Id: getdata.py,v 1.8 2023/06/06 22:50:14 sjg Exp $

	@(#) Copyright (c) 1996-2023 Simon J. Gerraty.

	This file is provided in the hope that it will
	be of use.  There is absolutely NO WARRANTY.
	Permission to copy, redistribute or otherwise
	use this file is hereby granted provided that 
	the above copyright notice and this notice are
	left intact. 
      
	Please send copies of changes and bug-fixes to:
	sjg@crufty.net

"""

import sys
import re
import subprocess
import dnsutils
import conf as cf

def get_dns_data(inf, conf={}):
    """extract records of interest from named_dump"""

    r = subprocess.Popen(['sort', '-r'], universal_newlines=True,
                         stdin=subprocess.PIPE, stdout=sys.stdout)

    # by default we grab all, use:
    # classess [=] in
    # to just select IN
    classes = conf.get('classes', [])
    if len(classes) > 0:
        class_re = re.compile('\s({})\s'.format('|'.join(classes)).lower())
    else:
        class_re = None

    want = conf.get('want_records', [])
    want_re = re.compile('\s({})\s'.format('|'.join(want)).lower())
    
    origin = ''
    lkey = None
    for line in inf.readlines():
        x = line.find(';')
        if x == 0:
            continue
        elif x > 0:
            line = line[:x]
        line = line.strip().lower()
        if line.startswith('$origin'):
            w = line.split()
            if w[1] == '.':
                origin = ''
            else:
                origin = '.'+w[1]
            continue
        if class_re:
            m = class_re.search(line)
            if not m:
                dnsutils.debug_print('get_dns_data', 'Skipping: {}'.format(line), conf)
                continue
        m = want_re.search(line)
        if not m:
            dnsutils.debug_print('get_dns_data', 'Skipping: {}'.format(line), conf)
            continue

        w = line.split()
        if 'nsec' in w or 'rrsig' in w:
            continue

        if 'in' in w:
            k = 3
        else:
            k = 2
        if w[0] == '.':
            key = w[0]
        elif w[0].endswith('.'):
            key = w[0].rstrip('.') + origin
        else:
            # key is same as previous record
            k -= 1
            key = lkey
        lkey = key
        if len(w) <= k:
            dnsutils.debug_print('get_dns_data', 'Unmatched: k={} {}'.format(k,w), conf)
            continue
        if w[k] == 'soa':
            print('SOA {} {} {}'.format(key,
                                        w[k+1].rstrip('.'),
                                        w[k+2].rstrip('.')), file=r.stdin)
            continue
        elif w[k] == 'mx':
            print('MX {} {} {}'.format(key, w[k+2].rstrip('.'), w[k+1]), file=r.stdin)
            continue
        elif w[k] in want:
            if w[k] == 'txt':
                data = ' '.join(w[k+1:])
            else:
                data = w[k+1].rstrip('.')
            print('{} {} {}'.format(w[k].upper(), key, data), file=r.stdin)
            continue

        dnsutils.debug_print('get_dns_data', 'Unmatched: k={} {}'.format(k,w), conf)


def myopts(o, a, conf):
    if o == '-X':
        conf['want_records'].append(a.lower())
        
if __name__ == '__main__':

    conf = {
        'want_records': ['a','aaaa','cname','mx','ns','ptr','soa'],
        }
    conf,args = dnsutils.app_init('X:', myopts, conf)

    get_dns_data(open(args[0], 'r'), conf)
    sys.exit(0)
